@@ -36,7 +36,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp,v 1.19 2004/02/12 19:13:52 chpe Exp $
+ * $Id: EphyHeaderSniffer.cpp,v 1.24 2004/03/05 00:29:00 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,6 +67,9 @@
 #include "nsIMIMEInfo.h"
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDownload.h"
+#if MOZILLA_SNAPSHOT > 10
+#include  "nsIMIMEHeaderParam.h"
+#endif
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -83,12 +86,12 @@ EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmb
 {
 	mPrompt = do_GetService("@mozilla.org/embedcomp/prompt-service;1");
 
-	LOG ("EphyHeaderSniffer constructor")
+	LOG ("EphyHeaderSniffer ctor (%p)", this)
 }
 
 EphyHeaderSniffer::~EphyHeaderSniffer()
 {
-	LOG ("EphyHeaderSniffer destructor")
+	LOG ("EphyHeaderSniffer dtor (%p)", this)
 }
 
 NS_IMPL_ISUPPORTS2(EphyHeaderSniffer, nsIWebProgressListener, nsIAuthPrompt)
@@ -242,6 +245,36 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 	if (defaultFileName.IsEmpty() && !mContentDisposition.IsEmpty())
 	{
 		/* 1 Use the HTTP header suggestion. */
+#if MOZILLA_SNAPSHOT > 10
+		nsCOMPtr<nsIMIMEHeaderParam> mimehdrpar =
+			do_GetService("@mozilla.org/network/mime-hdrparam;1");     
+		
+		if (mimehdrpar)
+		{
+			nsCAutoString fallbackCharset;
+			if (mURL)
+			{
+				mURL->GetOriginCharset(fallbackCharset);
+			}
+			
+			nsAutoString fileName;
+			
+			rv = mimehdrpar->GetParameter (mContentDisposition, "filename",
+						       fallbackCharset, PR_TRUE, nsnull,
+						       fileName);
+			if (NS_FAILED(rv) || fileName.IsEmpty())
+			{
+				rv = mimehdrpar->GetParameter (mContentDisposition, "name",
+							       fallbackCharset, PR_TRUE, nsnull,
+							       fileName);
+			}
+
+			if (NS_SUCCEEDED(rv) && !fileName.IsEmpty())
+			{
+				defaultFileName = fileName;
+			}
+		}
+#else
 		PRInt32 index = mContentDisposition.Find("filename=");
 		if (index >= 0)
 		{
@@ -251,6 +284,7 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 			mContentDisposition.Right(filename, mContentDisposition.Length() - index);
 			defaultFileName = NS_ConvertUTF8toUCS2(filename);
 		}
+#endif
 	}
     
 	if (defaultFileName.IsEmpty())
@@ -262,6 +296,9 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 		{
 			nsCAutoString fileNameCString;
 			url->GetFileName(fileNameCString);
+			/* FIXME: when we can depend on moz >= 1.5, use
+			 * CopyUTF8toUTF16 instead
+			 */
 			defaultFileName = NS_ConvertUTF8toUCS2(fileNameCString);
 		}
 	}
@@ -282,6 +319,9 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 		/* 4 Use the host. */
 		nsCAutoString hostName;
 		mURL->GetHost(hostName);
+		/* FIXME: when we can depend on moz >= 1.5, use
+		 * CopyUTF8toUTF16 instead
+		 */
 		defaultFileName = NS_ConvertUTF8toUCS2(hostName);
 	}
     
@@ -292,17 +332,23 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 	}
         
 	/* Validate the file name to ensure legality. */
-	for (PRUint32 i = 0; i < defaultFileName.Length(); i++)
-	{
-		if (defaultFileName[i] == ':' || defaultFileName[i] == '/')
-		{
-			defaultFileName.SetCharAt(i, PRUnichar(' '));
-		}
-	}
+	char *default_name = g_strdup (NS_ConvertUCS2toUTF8 (defaultFileName).get());
+	default_name = g_strdelimit (default_name, "/", ' ');
 
 	const char *key;
 	key = ephy_embed_persist_get_persist_key (EPHY_EMBED_PERSIST (mEmbedPersist));
 
+        char *filename;
+        filename = gnome_vfs_unescape_string (default_name, NULL);
+
+        if (!g_utf8_validate (filename, -1, NULL))
+        {
+                g_free (filename);
+                filename = g_strdup (default_name);
+        }
+
+	g_free (default_name);
+
 	if (askDownloadDest)
 	{
 		EphyFileChooser *dialog;
@@ -319,19 +365,21 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 						key ? key : CONF_STATE_SAVE_DIR);
 
 		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
-						   NS_ConvertUCS2toUTF8 (defaultFileName).get());
+                                                   filename);
 
 		g_signal_connect (dialog, "response",
 				  G_CALLBACK (filechooser_response_cb), this);
 
 		gtk_widget_show (GTK_WIDGET (dialog));
 
+                g_free (filename);
 		return NS_OK;
 	}
 	/* FIXME: how to inform user of failed save ? */
 
 	nsCOMPtr<nsILocalFile> destFile;
-	BuildDownloadPath (NS_ConvertUCS2toUTF8 (defaultFileName).get(), getter_AddRefs (destFile));
+	BuildDownloadPath (filename, getter_AddRefs (destFile));
+        g_free (filename);
 	NS_ENSURE_TRUE (destFile, NS_ERROR_FAILURE);
 
 	return InitiateDownload (destFile);
@@ -342,7 +390,7 @@ nsresult EphyHeaderSniffer::InitiateDownload (nsILocalFile *aDestFile)
 	LOG ("Initiating download")
 	return InitiateMozillaDownload (mDocument, mURL, aDestFile,
 					mContentType.get(), mOriginalURI, mEmbedPersist,
-					mPostData, nsnull);
+					mPostData, nsnull, -1);
 }
 
 NS_IMETHODIMP EphyHeaderSniffer::Prompt (const PRUnichar *dialogTitle, const PRUnichar *text,
