@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2001,2002,2003 Philip Langdale
+ *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,10 +15,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: MozRegisterComponents.cpp,v 1.21 2004/02/12 15:57:47 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include "ContentHandler.h"
@@ -25,19 +28,33 @@
 #include "GlobalHistory.h"
 #include "ExternalProtocolHandlers.h"
 #include "PrintingPromptService.h"
-#include "ProgressListener.h"
+#include "MozDownload.h"
 #include "ExternalProtocolService.h"
 #include "EphyAboutRedirector.h"
+#include "EphyContentPolicy.h"
 
+#ifdef HAVE_MOZILLA_PSM
+#include "GtkNSSClientAuthDialogs.h"
+#include "GtkNSSDialogs.h"
+#include "GtkNSSKeyPairDialogs.h"
+#endif
+
+#if MOZILLA_SNAPSHOT > 13
+#include <nsDocShellCID.h>
+#endif
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
 #include <nsCOMPtr.h>
 #include <nsILocalFile.h>
+#include <nsNetCID.h>
+#if MOZILLA_SNAPSHOT > 13
+#include <nsDocShellCID.h>
+#endif
 
 #include <glib.h>
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(EphyAboutRedirector)
-NS_GENERIC_FACTORY_CONSTRUCTOR(GProgressListener)
+NS_GENERIC_FACTORY_CONSTRUCTOR(MozDownload)	
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFilePicker)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GContentHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
@@ -45,11 +62,19 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GNewsProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(EphyContentPolicy)
 
 #if MOZILLA_SNAPSHOT < 12
 NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
 #endif
 
+#ifdef HAVE_MOZILLA_PSM
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSClientAuthDialogs)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSDialogs)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GtkNSSKeyPairDialogs)
+#endif
+
 static const nsModuleComponentInfo sAppComps[] = {
 #if MOZILLA_SNAPSHOT < 12
 	{
@@ -60,16 +85,10 @@ static const nsModuleComponentInfo sAppComps[] = {
 	},
 #endif
 	{
-		G_PROGRESSDIALOG_CLASSNAME,
-		G_PROGRESSDIALOG_CID,
-		G_PROGRESSDIALOG_CONTRACTID,
-		GProgressListenerConstructor
-	},
-	{
-		G_PROGRESSDIALOG_CLASSNAME,
-		G_PROGRESSDIALOG_CID,
+		MOZ_DOWNLOAD_CLASSNAME,
+		MOZ_DOWNLOAD_CID,
 		NS_DOWNLOAD_CONTRACTID,
-		GProgressListenerConstructor
+		MozDownloadConstructor
 	},
 	{
 		G_FILEPICKER_CLASSNAME,
@@ -77,6 +96,32 @@ static const nsModuleComponentInfo sAppComps[] = {
 		G_FILEPICKER_CONTRACTID,
 		GFilePickerConstructor
 	},
+#ifdef HAVE_MOZILLA_PSM
+	{
+		GTK_NSSCLIENTAUTHDIALOGS_CLASSNAME,
+		GTK_NSSCLIENTAUTHDIALOGS_CID,
+		NS_CLIENTAUTHDIALOGS_CONTRACTID,
+		GtkNSSClientAuthDialogsConstructor
+	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_BADCERTLISTENER_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
+	{
+		GTK_NSSDIALOGS_CLASSNAME,
+		GTK_NSSDIALOGS_CID,
+		NS_CERTIFICATEDIALOGS_CONTRACTID,
+		GtkNSSDialogsConstructor
+	},
+	{
+		GTK_NSSKEYPAIRDIALOGS_CLASSNAME,
+		GTK_NSSKEYPAIRDIALOGS_CID,
+		NS_GENERATINGKEYPAIRINFODIALOGS_CONTRACTID,
+		GtkNSSKeyPairDialogsConstructor
+	},
+#endif
 	{
 		NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
 		G_CONTENTHANDLER_CID,
@@ -86,7 +131,11 @@ static const nsModuleComponentInfo sAppComps[] = {
 	{
 		EPHY_GLOBALHISTORY_CLASSNAME,
 		EPHY_GLOBALHISTORY_CID,
+#if MOZILLA_SNAPSHOT > 13
+		NS_GLOBALHISTORY2_CONTRACTID,
+#else
 		NS_GLOBALHISTORY_CONTRACTID,
+#endif
 		MozGlobalHistoryConstructor
 	},
 	{
@@ -131,12 +180,6 @@ static const nsModuleComponentInfo sAppComps[] = {
 		EPHY_ABOUT_REDIRECTOR_EPIPHANY_CONTRACTID,
 		EphyAboutRedirectorConstructor
 	},
-	{
-		EPHY_ABOUT_REDIRECTOR_CLASSNAME,
-		EPHY_ABOUT_REDIRECTOR_CID,
-		EPHY_ABOUT_REDIRECTOR_OPTIONS_CONTRACTID,
-		EphyAboutRedirectorConstructor
-	},
 	{
 		EPHY_ABOUT_REDIRECTOR_CLASSNAME,
 		EPHY_ABOUT_REDIRECTOR_CID,
@@ -148,21 +191,7 @@ static const nsModuleComponentInfo sAppComps[] = {
 		EPHY_ABOUT_REDIRECTOR_CID,
 		EPHY_ABOUT_REDIRECTOR_MARCO_CONTRACTID,
 		EphyAboutRedirectorConstructor
-	}
-};
-
-static const int sNumAppComps = sizeof(sAppComps) / sizeof(nsModuleComponentInfo);
-
-static const nsModuleComponentInfo sFtpComps = {
-	G_FTP_PROTOCOL_CLASSNAME,
-	G_FTP_PROTOCOL_CID,
-	G_FTP_PROTOCOL_CONTRACTID,
-	GFtpProtocolHandlerConstructor
-};
-
-NS_GENERIC_FACTORY_CONSTRUCTOR(GMailtoProtocolHandler)
-
-static const nsModuleComponentInfo sMailtoComps[] = {
+	},
 	{
 		G_MAILTO_PROTOCOL_CLASSNAME,
 		G_MAILTO_PROTOCOL_CID,
@@ -174,17 +203,22 @@ static const nsModuleComponentInfo sMailtoComps[] = {
 		G_MAILTO_PROTOCOL_CID,
 		G_MAILTO_CONTENT_CONTRACTID,
 		GMailtoProtocolHandlerConstructor
+	},
+	{
+		EPHY_CONTENT_POLICY_CLASSNAME,
+		EPHY_CONTENT_POLICY_CID,
+		EPHY_CONTENT_POLICY_CONTRACTID,
+		EphyContentPolicyConstructor
 	}
 };
 
-static const int sNumMailtoComps = sizeof(sMailtoComps) / sizeof(nsModuleComponentInfo);
+static const int sNumAppComps = sizeof(sAppComps) / sizeof(nsModuleComponentInfo);
 
-static const nsModuleComponentInfo sModuleComps[] = {
-	{
-		 G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
-		 G_EXTERNALPROTOCOLSERVICE_CID,
-		 NS_EXTERNALPROTOCOLSERVICE_CONTRACTID
-	}
+static const nsModuleComponentInfo sFtpComps = {
+	G_FTP_PROTOCOL_CLASSNAME,
+	G_FTP_PROTOCOL_CID,
+	G_FTP_PROTOCOL_CONTRACTID,
+	GFtpProtocolHandlerConstructor
 };
 
 static NS_DEFINE_CID(knsFtpProtocolHandlerCID, NS_FTPPROTOCOLHANDLER_CID);
@@ -200,16 +234,18 @@ mozilla_register_components (void)
 	nsresult rv;
 
 	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	NS_ENSURE_SUCCESS(rv, rv);
+	NS_GetComponentRegistrar(getter_AddRefs(cr));
+	NS_ENSURE_TRUE (cr, FALSE);
 
 	for (int i = 0; i < sNumAppComps; i++)
 	{
 		nsCOMPtr<nsIGenericFactory> componentFactory;
 		rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
 					  &(sAppComps[i]));
-		if (NS_FAILED(rv))
+		if (NS_FAILED(rv) || !componentFactory)
 		{
+			g_warning ("Failed to make a factory for %s\n", sAppComps[i].mDescription);
+
 			ret = FALSE;
 			continue;  // don't abort registering other components
 		}
@@ -219,7 +255,11 @@ mozilla_register_components (void)
 					 sAppComps[i].mContractID,
 					 componentFactory);
 		if (NS_FAILED(rv))
+		{
+			g_warning ("Failed to register %s\n", sAppComps[i].mDescription);
+
 			ret = FALSE;
+		}
 	}
 
 	return ret;
@@ -233,34 +273,34 @@ mozilla_register_FtpProtocolHandler (void)
 {
 	if (ftpRegistered == PR_TRUE) return TRUE;
 
-	nsresult rv = NS_OK;
+	nsresult rv;
 
 	nsCOMPtr<nsIComponentManager> cm;
-	rv = NS_GetComponentManager(getter_AddRefs(cm));
-	if (NS_FAILED(rv) || !cm) return FALSE;
+	NS_GetComponentManager(getter_AddRefs(cm));
+	NS_ENSURE_TRUE (cm, FALSE);
 
 	rv = cm->GetClassObject(knsFtpProtocolHandlerCID,
 				NS_GET_IID(nsIFactory),
 				getter_AddRefs(nsFtpFactory));
-        if (NS_FAILED(rv)) return FALSE;
+	if (NS_FAILED (rv)) return FALSE;
 
 	nsCOMPtr<nsIGenericFactory> ftpFactory;
-	rv = NS_NewGenericFactory(getter_AddRefs(ftpFactory),
-				  &sFtpComps);
-	if (NS_FAILED(rv) || !ftpFactory) return FALSE;
+	NS_NewGenericFactory(getter_AddRefs(ftpFactory), &sFtpComps);
+	NS_ENSURE_TRUE (ftpFactory, FALSE);
 
 	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
+	NS_GetComponentRegistrar(getter_AddRefs(cr));
+	NS_ENSURE_TRUE (cr, FALSE);
 
 	rv = cr->RegisterFactory(sFtpComps.mCID,
 				 sFtpComps.mDescription,
 				 sFtpComps.mContractID,
 				 ftpFactory);  
-	if (NS_FAILED(rv)) return FALSE;
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
 	ftpRegistered = PR_TRUE;
-	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+
+	return TRUE;
 }
 
 /**
@@ -271,51 +311,18 @@ mozilla_unregister_FtpProtocolHandler (void)
 {
 	if (ftpRegistered == PR_FALSE) return FALSE;
         
-        nsresult rv = NS_OK;
-
+        nsresult rv;
 	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
+	NS_GetComponentRegistrar(getter_AddRefs(cr));
+	NS_ENSURE_TRUE (cr, FALSE);
 
 	rv = cr->RegisterFactory(knsFtpProtocolHandlerCID,
 				 NS_FTPPROTOCOLHANDLER_CLASSNAME,
 				 G_FTP_PROTOCOL_CONTRACTID,
 				 nsFtpFactory);
+	NS_ENSURE_SUCCESS (rv, FALSE);
 
 	ftpRegistered = PR_FALSE;
-        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
-}
 
-/**
- * mozilla_register_MailtoProtocolHandler: Register Mailto Protocol Handler
- */
-gboolean 
-mozilla_register_MailtoProtocolHandler (void)
-{
-	gboolean retVal = TRUE;
-        nsresult rv;
-
-	nsCOMPtr<nsIComponentRegistrar> cr;
-	rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
-	if (NS_FAILED(rv) || !cr) return FALSE;
-
-	for (int i = 0; i < sNumMailtoComps; i++)
-	{
-		nsCOMPtr<nsIGenericFactory> componentFactory;
-		rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
-					  &(sMailtoComps[i]));
-		if (NS_FAILED(rv))
-		{
-			retVal = FALSE;
-			continue;  // don't abort registering other components
-		}
-
-		rv = cr->RegisterFactory(sMailtoComps[i].mCID,
-					 sMailtoComps[i].mDescription,
-					 sMailtoComps[i].mContractID,
-					 componentFactory);
-		if (NS_FAILED(rv))
-			retVal = FALSE;
-	}
-	return retVal;
+        return TRUE;
 }
