@@ -16,11 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-event.cpp,v 1.1 2004/01/15 18:08:24 chpe Exp $
+ *  $Id: mozilla-embed-event.cpp,v 1.3 2004/02/29 13:23:28 chpe Exp $
  */
 
 #include "mozilla-embed-event.h"
 
+#include "ephy-debug.h"
+
 #include <nsCOMPtr.h>
 #include <nsIDOMEvent.h>
 
@@ -37,7 +39,7 @@ struct MozillaEmbedEventPrivate
 
 static void mozilla_embed_event_class_init	(MozillaEmbedEventClass *klass);
 static void mozilla_embed_event_init		(MozillaEmbedEvent *event);
-static void ephy_embed_event_iface_init		(EphyEmbedEventIFace *iface);
+static void ephy_embed_event_iface_init		(EphyEmbedEventIface *iface);
 
 static GObjectClass *parent_class = NULL;
 
@@ -97,6 +99,10 @@ mozilla_embed_event_set_property (MozillaEmbedEvent *event,
 				  const char *name,
 				  GValue *value)
 {
+	char *value_content = g_strdup_value_contents (value);
+	LOG ("embed event %p set property \"%s\" to %s", event, name, value_content);
+	g_free (value_content);
+
 	g_hash_table_insert (event->priv->props,
 			     g_strdup (name),
 			     value);
@@ -166,6 +172,8 @@ mozilla_embed_event_init (MozillaEmbedEvent *event)
 {
 	event->priv = MOZILLA_EMBED_EVENT_GET_PRIVATE (event);
 
+	LOG ("MozillaEmbedEvent %p initialising", event)
+
 	event->priv->dom_event = nsnull;
 	event->priv->props = g_hash_table_new_full (g_str_hash, g_str_equal,
 						    g_free, free_g_value);
@@ -180,11 +188,13 @@ mozilla_embed_event_finalize (GObject *object)
 
 	event->priv->dom_event = nsnull;
 
+	LOG ("MozillaEmbedEvent %p finalised", object)
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
-ephy_embed_event_iface_init (EphyEmbedEventIFace *iface)
+ephy_embed_event_iface_init (EphyEmbedEventIface *iface)
 {
 	iface->get_type = impl_get_type;
 	iface->get_context = impl_get_context;
