@@ -130,8 +130,16 @@ ephy_state_window_set_size (GtkWidget *window, EphyNode *node)
 
 	if (size)
 	{
-		gtk_window_set_default_size
-			(GTK_WINDOW (window), width, height);
+		GdkScreen *screen;
+		int screen_width, screen_height;
+
+		screen = gdk_screen_get_default ();
+		screen_width = gdk_screen_get_width (screen);
+		screen_height = gdk_screen_get_height (screen);
+
+		gtk_window_set_default_size (GTK_WINDOW (window),
+					     MIN (width, screen_width),
+					     MIN (height, screen_height));
 	}
 
 	if (maximize)
@@ -157,7 +165,7 @@ ephy_state_window_set_position (GtkWidget *window, EphyNode *node)
 	size = ephy_node_get_property_boolean (node, EPHY_NODE_STATE_PROP_POSITION);
 
 	/* Don't set the position of the window if it is maximized */
-	if (!maximize & size)
+	if ((!maximize) && size)
 	{
 		x = ephy_node_get_property_int (node, EPHY_NODE_STATE_PROP_POSITION_X);
 		y = ephy_node_get_property_int (node, EPHY_NODE_STATE_PROP_POSITION_Y);
