@@ -1,5 +1,7 @@
 /*
  *  Copyright (C) 2001, 2002 Ricardo Fernández Pascual
+ *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,22 +17,25 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-nautilus-view.c,v 1.17.2.1 2003/12/14 17:30:12 chpe Exp $
+ *  $Id: ephy-nautilus-view.c,v 1.31 2004/02/24 17:43:48 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
-#include <config.h>
 #include <libgnome/gnome-macros.h>
 #include <bonobo/bonobo-zoomable.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <string.h>
 
+#include "ephy-embed-factory.h"
 #include "ephy-embed-popup-control.h"
 #include "ephy-nautilus-view.h"
 #include "ephy-embed.h"
-#include "ephy-embed-utils.h"
 #include "find-dialog.h"
 #include "print-dialog.h"
+#include "ephy-encoding-dialog.h"
 #include "ephy-zoom.h"
 #include "ephy-debug.h"
 
@@ -44,7 +49,6 @@ static void		gnv_embed_new_window_cb			(EphyEmbed *embed,
 								 EmbedChromeMask chromemask,
 								 EphyNautilusView *view);
 static void		gnv_embed_link_message_cb 		(EphyEmbed *embed, 
-								 const char *message,
 								 EphyNautilusView *view);
 static gint		gnv_embed_dom_mouse_click_cb		(EphyEmbed *embed,
 								 EphyEmbedEvent *event,
@@ -79,16 +83,15 @@ static void		gnv_zoomable_zoom_to_fit_cb		(BonoboZoomable *zoomable,
 static void		gnv_zoomable_zoom_to_default_cb		(BonoboZoomable *zoomable,
 								 EphyNautilusView *view);
 /* commands */
-static void		gnv_cmd_set_encoding			(BonoboUIComponent *uic, 
-								 EphyNautilusView *view,
-								 const char* verbname);
 static void 		gnv_cmd_file_print			(BonoboUIComponent *uic, 
 								 EphyNautilusView *view, 
 								 const char* verbname);
 static void 		gnv_cmd_edit_find			(BonoboUIComponent *uic, 
 								 EphyNautilusView *view, 
 								 const char* verbname);
-
+static void 		gnv_cmd_select_encoding			(BonoboUIComponent *uic, 
+								 EphyNautilusView *view, 
+								 const char* verbname);
 
 /* popups */
 static EphyNautilusView *gnv_view_from_popup			(EphyEmbedPopupControl*popup);
@@ -103,6 +106,8 @@ static void 		gnv_popup_cmd_frame_in_new_window	(BonoboUIComponent *uic,
 								 EphyEmbedPopupControl*popup, 
 								 const char* verbname);
 
+#define EPHY_NAUTILUS_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NAUTILUS_VIEW, EphyNautilusViewPrivate))
+
 struct EphyNautilusViewPrivate {
 	EphyEmbed *embed;
 	char *title;
@@ -129,40 +134,49 @@ static BonoboUIVerb ephy_popup_verbs [] = {
 BonoboUIVerb ephy_verbs [] = {
         BONOBO_UI_VERB ("FilePrint", (BonoboUIVerbFn) gnv_cmd_file_print),
         BONOBO_UI_VERB ("EditFind", (BonoboUIVerbFn) gnv_cmd_edit_find),
+	BONOBO_UI_VERB ("ViewEncoding", (BonoboUIVerbFn) gnv_cmd_select_encoding),
         BONOBO_UI_VERB_END
 };
 
-#define ENCODING_MENU_PATH "/menu/View/Encoding"
-
-
 BONOBO_CLASS_BOILERPLATE (EphyNautilusView, ephy_nautilus_view,
 			  NautilusView, NAUTILUS_TYPE_VIEW)
 
+static gboolean
+disconnected_idle (EphyShell *shell)
+{
+	g_object_unref (shell);
+
+	return FALSE;
+}
+
+static void
+control_disconnected_cb (BonoboControl *control)
+{
+	g_idle_add ((GSourceFunc)disconnected_idle, ephy_shell);
+}
+
 static void
 ephy_nautilus_view_instance_init (EphyNautilusView *view)
 {
 	GtkWidget *w;
-	EphyNautilusViewPrivate *p = g_new0 (EphyNautilusViewPrivate, 1);
-	EphyEmbedSingle *single;
+	EphyNautilusViewPrivate *p = EPHY_NAUTILUS_VIEW_GET_PRIVATE (view);
 	float *levels;
 	gchar **names;
 	guint i;
-
-	single = ephy_embed_shell_get_embed_single
-		(EPHY_EMBED_SHELL (ephy_shell));
+	BonoboControl *control;
 
 	view->priv = p;
-	view->priv->embed = ephy_embed_new (G_OBJECT (single));
 
-	g_object_ref (G_OBJECT (ephy_shell));
+	view->priv->embed = EPHY_EMBED
+		(ephy_embed_factory_new_object ("EphyEmbed"));
 
-	g_signal_connect (view->priv->embed, "ge_link_message",
+	g_signal_connect (view->priv->embed, "link_message",
 			  G_CALLBACK (gnv_embed_link_message_cb), 
 			  view);
 	g_signal_connect (view->priv->embed, "ge_location",
 			  G_CALLBACK (gnv_embed_location_cb), 
 			  view);
-	g_signal_connect (view->priv->embed, "ge_title",
+	g_signal_connect (view->priv->embed, "title",
 			  G_CALLBACK (gnv_embed_title_cb), 
 			  view);
 	g_signal_connect (view->priv->embed, "ge_new_window",
@@ -189,7 +203,12 @@ ephy_nautilus_view_instance_init (EphyNautilusView *view)
 	g_signal_connect (G_OBJECT (view), "stop_loading",
 			  G_CALLBACK (gnv_stop_loading_cb), NULL);
 
-        g_signal_connect (G_OBJECT (nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view))),
+	control = nautilus_view_get_bonobo_control (NAUTILUS_VIEW (view));	
+	g_object_ref (ephy_shell);
+
+	g_signal_connect (control, "disconnected",
+			  G_CALLBACK (control_disconnected_cb), NULL);
+        g_signal_connect (control,
                           "activate",
                           G_CALLBACK (gnv_bonobo_control_activate_cb), view);
 
@@ -247,15 +266,10 @@ ephy_nautilus_view_instance_init (EphyNautilusView *view)
 	bonobo_ui_component_add_verb_list_with_data (p->popup_ui, ephy_popup_verbs, p->popup);
 }
 
-/**
- * Returns a new EphyNautilusView as a BonoboObject 
- **/
 BonoboObject *
-ephy_nautilus_view_new_component (EphyShell *gs)
+ephy_nautilus_view_new_component (EphyShell *shell)
 {
-	EphyNautilusView *view;
-	view = EPHY_NAUTILUS_VIEW (g_object_new (EPHY_TYPE_NAUTILUS_VIEW, NULL));
-	return BONOBO_OBJECT (view);
+	return BONOBO_OBJECT (g_object_new (EPHY_TYPE_NAUTILUS_VIEW, NULL));
 }
 
 static void
@@ -275,17 +289,18 @@ ephy_nautilus_view_finalize (GObject *object)
 
 	g_free (p->title);
 	g_free (p->location);
-	g_free (p);
 
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
-
-	g_object_unref (G_OBJECT (ephy_shell));
 }
 
 static void
 ephy_nautilus_view_class_init (EphyNautilusViewClass *class)
 {
-	G_OBJECT_CLASS (class)->finalize = ephy_nautilus_view_finalize;
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+	object_class->finalize = ephy_nautilus_view_finalize;
+
+	g_type_class_add_private (object_class, sizeof(EphyNautilusViewPrivate));
 }
 
 static gint
@@ -296,8 +311,8 @@ gnv_embed_dom_mouse_click_cb (EphyEmbed *embed,
 	EphyEmbedEventType type;
 	EmbedEventContext context;
 
-	ephy_embed_event_get_event_type (event, &type);
-	ephy_embed_event_get_context (event, &context);
+	type = ephy_embed_event_get_event_type (event);
+	context = ephy_embed_event_get_context (event);
 
 	if (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2
 	    && (context & EMBED_CONTEXT_LINK))
@@ -309,8 +324,9 @@ gnv_embed_dom_mouse_click_cb (EphyEmbed *embed,
 
 		g_return_val_if_fail (url, FALSE);
 
-		nautilus_view_open_location_force_new_window (NAUTILUS_VIEW (view),
-							      url, NULL);
+		nautilus_view_open_location (NAUTILUS_VIEW (view), url,
+				             Nautilus_ViewFrame_OPEN_IN_NAVIGATION,
+				             0, NULL);
 	}
 
 	return FALSE;
@@ -322,20 +338,23 @@ gnv_embed_context_menu_cb (EphyEmbed *embed,
 			   EphyNautilusView *view)
 {
 	EphyNautilusViewPrivate *p = view->priv;
-	EmbedEventContext context;
 
-	ephy_embed_event_get_context (event, &context);
 	ephy_embed_popup_control_set_event (p->popup, event);
 	ephy_embed_popup_control_show (p->popup, embed);
 }
 
 static void
-gnv_embed_link_message_cb (EphyEmbed *embed, const char *message, EphyNautilusView *view)
+gnv_embed_link_message_cb (EphyEmbed *embed, EphyNautilusView *view)
 {
+	char *message;
+
 	g_return_if_fail (EPHY_IS_NAUTILUS_VIEW (view));
-	g_return_if_fail (message != NULL);
+
+	message = ephy_embed_get_link_message (embed);
 
 	nautilus_view_report_status (NAUTILUS_VIEW (view), message);
+
+	g_free (message);
 }
 
 static void
@@ -379,7 +398,7 @@ gnv_embed_title_cb (EphyEmbed *embed, EphyNautilusView *view)
 	g_return_if_fail (view->priv->embed == embed);
 
 	g_free (p->title);
-	ephy_embed_get_title (embed, &p->title);
+	p->title = ephy_embed_get_title (embed);
 
  	nautilus_view_set_title (NAUTILUS_VIEW (view), p->title);
 }
@@ -408,19 +427,14 @@ gnv_embed_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
 	EphyWindow *window;
 	
 	window = ephy_window_new ();
-	if (!(chromemask & EMBED_CHROME_OPENASCHROME))
-	{
-		ephy_window_set_chrome (window, 
-					  chromemask | 
-					  EMBED_CHROME_OPENASPOPUP);
-	}
 	new_tab = ephy_tab_new ();
+	gtk_widget_show (GTK_WIDGET (new_tab));
+
 	ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_LAST, FALSE);
 
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
 
-
 static void
 gnv_bonobo_control_activate_cb (BonoboControl *control, gboolean state, EphyNautilusView *view)
 {
@@ -431,11 +445,6 @@ gnv_bonobo_control_activate_cb (BonoboControl *control, gboolean state, EphyNaut
 		p->ui = nautilus_view_set_up_ui (NAUTILUS_VIEW (view), SHARE_DIR,
 						 "nautilus-epiphany-view.xml", "EphyNautilusView");
 		g_return_if_fail (BONOBO_IS_UI_COMPONENT (p->ui));
-		
-		ephy_embed_utils_build_encodings_submenu (p->ui,
-							   ENCODING_MENU_PATH,
-							   (BonoboUIVerbFn) gnv_cmd_set_encoding,
-							   view);
 
 		bonobo_ui_component_add_verb_list_with_data (p->ui, ephy_verbs, view);
 	}
@@ -463,8 +472,10 @@ gnv_popup_cmd_new_window (BonoboUIComponent *uic,
 	
 	ephy_embed_event_get_property (info, "link", &value);
 
-	nautilus_view_open_location_force_new_window (NAUTILUS_VIEW (view),
-						      g_value_get_string (value), NULL);
+	nautilus_view_open_location (NAUTILUS_VIEW (view),
+				     g_value_get_string (value),
+				     Nautilus_ViewFrame_OPEN_IN_NAVIGATION,
+				     0, NULL);
 }
 
 static void 
@@ -482,8 +493,10 @@ gnv_popup_cmd_image_in_new_window (BonoboUIComponent *uic,
 	
 	ephy_embed_event_get_property (info, "image", &value);
 
-	nautilus_view_open_location_force_new_window (NAUTILUS_VIEW (view),
-						      g_value_get_string (value), NULL);
+	nautilus_view_open_location (NAUTILUS_VIEW (view),
+				     g_value_get_string (value),
+				     Nautilus_ViewFrame_OPEN_IN_NAVIGATION,
+				     0, NULL);
 }
 
 static void 
@@ -499,30 +512,30 @@ gnv_popup_cmd_frame_in_new_window (BonoboUIComponent *uic,
 	
 	info = ephy_embed_popup_control_get_event (popup);
 	
-	ephy_embed_get_location (view->priv->embed, FALSE, &location);
+	location = ephy_embed_get_location (view->priv->embed, FALSE);
 
-	nautilus_view_open_location_force_new_window (NAUTILUS_VIEW (view),
-						      location, NULL);
+	nautilus_view_open_location (NAUTILUS_VIEW (view),
+				     location,
+				     Nautilus_ViewFrame_OPEN_IN_NAVIGATION,
+				     0, NULL);
+	
 	g_free (location);
 }
 
-void 
-gnv_cmd_set_encoding (BonoboUIComponent *uic, 
-		      EphyNautilusView *view,
-		      const char* verbname)
+static void
+gnv_cmd_select_encoding (BonoboUIComponent *uic, 
+			 EphyNautilusView *view, 
+			 const char* verbname)
 {
-	const char *encoding;
+	EphyDialog *dialog;
+	
+	dialog = EPHY_DIALOG (g_object_new (EPHY_TYPE_ENCODING_DIALOG,
+					    "embed", view->priv->embed,
+					    NULL));
 
-	g_return_if_fail (EPHY_IS_NAUTILUS_VIEW (view));
-
-	if (strncmp (verbname, "Encoding", 8) == 0)
-	{
-		encoding = verbname + 8;
-		
-		LOG ("Set encoding %s", encoding)
-
-		ephy_embed_set_encoding (view->priv->embed, encoding);
-	}
+	ephy_dialog_set_modal (dialog, TRUE);
+	ephy_dialog_run (dialog);
+	g_object_unref (dialog);
 }
 
 static void
@@ -533,10 +546,11 @@ gnv_cmd_file_print (BonoboUIComponent *uic,
 	EphyDialog *dialog;
 	EphyNautilusViewPrivate *p = view->priv;
 	
-	dialog = print_dialog_new (p->embed, NULL);
+	dialog = ephy_print_dialog_new (NULL, p->embed, FALSE);
 
 	ephy_dialog_set_modal (dialog, TRUE);
-	ephy_dialog_show (dialog);
+	ephy_dialog_run (dialog);
+	g_object_unref (dialog);
 }
 
 static void
@@ -556,7 +570,6 @@ gnv_cmd_edit_find (BonoboUIComponent *uic,
 	ephy_dialog_show (p->find_dialog);
 }
 
-
 /* zoomable */
 static void
 gnv_zoomable_set_zoom_level_cb (BonoboZoomable *zoomable,
@@ -573,12 +586,10 @@ gnv_zoomable_zoom_in_cb (BonoboZoomable *zoomable,
 			 EphyNautilusView *view)
 {
 	float zoom, new_zoom;
-	gresult rv;
 	
 	g_return_if_fail (EPHY_IS_NAUTILUS_VIEW (view));
 
-	rv = ephy_embed_zoom_get (view->priv->embed, &zoom);
-	if (rv == G_FAILED) return;
+	zoom = ephy_embed_zoom_get (view->priv->embed);
 
 	new_zoom = ephy_zoom_get_changed_zoom_level (zoom, 1);
 	ephy_embed_zoom_set (view->priv->embed, new_zoom, TRUE);
@@ -589,12 +600,10 @@ gnv_zoomable_zoom_out_cb (BonoboZoomable *zoomable,
 			  EphyNautilusView *view)
 {
 	float zoom, new_zoom;
-	gresult rv;
 		
 	g_return_if_fail (EPHY_IS_NAUTILUS_VIEW (view));
 
-	rv = ephy_embed_zoom_get (view->priv->embed, &zoom);
-	if (rv == G_FAILED) return;
+	zoom = ephy_embed_zoom_get (view->priv->embed);
 
 	new_zoom = ephy_zoom_get_changed_zoom_level (zoom, -1);
 	ephy_embed_zoom_set (view->priv->embed, new_zoom, TRUE);
